-- playground
--

-- --------------------------------------------------------------------
-- serialice-tools.lua generic functions

require("bit")

function printf(s,...)
	return io.write(s:format(...))
end

function new(s, o)
	o = o or { } -- create object if user does not provide one
	setmetatable(o, s)
	s.__index = s
	return o
end

components = nil

function add(c)
	components = { next = components, component = c }
end

-- --------------------------------------------------------------------
-- SuperIO Winbond W83627 component

superio_w83627thg = {
	superio_base = 0x2e;
	catch_serial = false;
}

function superio_w83627thg:new(o)
	return new(self, o)
end

function superio_w83627thg:init(base, catch_serial)
	self.superio_base = base;
end

function superio_w83627thg:IO_read_filter(a,b)
    printf("Filter: Winbond %x, %x\n", a, self.superio_base);
end

-- --------------------------------------------------------------------
-- SuperIO SMSC something else component

superio_smsc = {
	superio_base = 0x2e;
}

function superio_smsc:new(o)
	return new(self, o)
end

function superio_smsc:init(base)
	self.superio_base = base;
end

function superio_smsc:IO_read_filter(a,b)
    printf("Filter: SMSC %x, %x\n", a, self.superio_base);
end

-- --------------------------------------------------------------------
-- of course 3 superios is stupid, but it's a simple example
-- add them all to our components list

first_superio = superio_w83627thg:new{superio_base=0x62}
add(first_superio)

second_superio = superio_smsc:new{}
add(second_superio)

third_superio = superio_smsc:new{superio_base=0x4e}
-- third_superio:init(0x48)
add(third_superio)


-- --------------------------------------------------------------------
-- a filter could look like this

local current_component = components

while current_component do
   current_component.component:IO_read_filter(23, 14)
   current_component = current_component.next
end


-- --------------------------------------------------------------------
-- sample output:
--
-- Filter: SMSC 17, 4e
-- Filter: SMSC 17, 2e
-- Filter: Winbond 17, 62
--
